# $Id: NYTimes.pm,v 1.8 2004/09/18 22:07:11 asc Exp $
use strict;

package ASCOPE::Article::Collection::NYTimes;
use base qw (ASCOPE::Article::Collection);

$ASCOPE::Article::Collection::NYTimes::VERSION = '1.0';

=head1 NAME

ASCOPE::Article::Collection::NYTimes - New York Times specific methods for ASCOPE::Article::Collection

=head1 SYNOPSIS

 use ASCOPE::Article::Collection::NYTimes;
 
 my $nyt  = ASCOPE::Article::Collection::NYTimes->new();
 my $data = $nyt->plot_location("Laos");

=head1 DESCRIPTION

New York Times specific methods for ASCOPE::Article::Collection

=cut

use Locale::Country;
use Geography::USStates;
use Memoize;

use ASCOPE::Article::Collection::NYTimes::Country;
use ASCOPE::Article::Collection::NYTimes::USA;
use ASCOPE::Article::Collection::NYTimes::CAN;
use ASCOPE::Article::Collection::NYTimes::Intl;
use ASCOPE::Article::Collection::NYTimes::LocationDB;

&memoize("_plot");

sub new {
    my $pkg = shift;

    my $self = $pkg->SUPER::new(@_);

    if (! $self) {
	return undef;
    }

    #

    $self->{'__db'} = undef;
    return bless $self, $pkg;
}

=head1 OBJECT METHODS

This package inherits object methods from ASCOPE::Article::Collection

=cut

=head2 $obj->load_location_db($path)

Load a list of previously-indexed locations. You can retrieve an 
existing index of 300+ locations from : 

 http://aaronland.info/nytimes/knows/where/places.sqlite

=cut

sub load_location_db {
    my $self = shift;
    my $path = shift;

    my $db_class = "ASCOPE::Article::Collection::NYTimes::LocationDB";

    if ($db_class->load($path)) {
	$self->{'__db'} = $db_class;
	return 1;
    }

    return 0;
}

=head2 $obj->plot_location($name)

Returns a hash reference, whose keys are :

 name
 latitude
 longitude
 locality
 region
 country

If a location database has been loaded, new entries will be
added.

=cut

sub plot_location {
    my $self = shift;
    my $name = shift;

    my $db = $self->{'__db'};

    if (($db) && (my ($res) = $self->find_location(name => $name))) {
	return { map {
	    $_ => $res->$_();
	} $res->columns() };
    }

    my $data = &_plot($name);

    if (($db) && ($data)) {
	$self->add_location($data);
    }

    return $data;
}

=head2 $obj->find_location(\%where,\%orderby)

Returns one, or more, ASCOPE::Article::Collection::NYTimes::LocationDB
objects or a Class::DBI::Iterator object.

=cut

sub find_location {
    my $self = shift;
    return $self->{'__db'}->search_where(@_);
}

=head2 $obj->add_location(\%data)

Returns a ASCOPE::Article::Collection::NYTimes::LocationDB object

=cut

sub add_location {
    my $self = shift;
    return $self->{'__db'}->create(@_);
}

sub _plot {
    my $name  = shift;

    #

    my $localname =  $name;
    $localname    =~ s/([\w\s])\sState$/$1/;
    
    my $test = "undef";

    my $code = undef;
    my $lat  = undef;
    my $long = undef;

    my $country  = undef;
    my $region   = undef;
    my $locality = undef;

    # Hack - need to submit patch
    # for Locale::Country

    if ($localname eq "Laos") {
	$localname = "Lao People's Democratic Republic";
    }

    elsif ($localname eq "Serbia") {
	$localname = "Serbia and Montenegro";
    }

    elsif ($localname eq "Georgia (Georgian Republic)") {
	$localname = "Georgia";
    }

    elsif ($localname eq "Ivory Coast") {
	$localname = "Cote D'Ivoire";
    }

    elsif ($localname eq "Bahama Islands") {
	$localname = "Bahamas";
    }

    elsif ($localname eq "Vatican City") {
	$localname = "Holy See ($localname)";
    }

    else {}

    #

    if ($code = country2code($localname, LOCALE_CODE_ALPHA_2)) {
	$test =  "country";

	$country     = uc($code);
	# print "[$localname] $country\n";

	($lat,$long) = &country_latlong($country);
    }
    
    elsif ($code = &getState($localname)) {
	$test =  "state (1)";

	$country = "US";
	$region  = uc($code);

	($lat,$long) = &usstate_latlong($region);
    }
    
    elsif ($localname =~ /Province \(Canada\)$/) {

	$test =  "province (1)";
	my @prov = ();
	
	foreach my $part (split(/\s/,$localname)) {
	    if ($part eq "Province") {
		last;
	    }
	    
	    else {
		push @prov, $part;
	    }
	}

	my $country = "CAN";
	my $region  = &cdnprovince_iso(join("",@prov));

	($lat,$long) = &cdnprovince_latlong($code);
    }

    elsif (($localname =~ /^([\w\s]+)\s\(State\)$/) &&
	   ($code = &getState($1))) {

	$test =  "state (2)";

	$country = "US";
	$region  = uc($code);
	
	($lat,$long) = &usstate_latlong($region);
    } 

    elsif ($localname =~ /^([\w\s]+)\s\((\w{2})\)$/) {

	$test =  "us city (1)";
	
	$country  = "US";
	$region   = $2;
	$locality = $1;
	
	($lat,$long) = &uscity_latlong($locality,$region);
    }

    elsif ($localname =~ /^([\w\s]+)\s\(NY(?:C|S Area)\)$/) {

	$country  = "US";
	$region   = "NY";
	$locality = $1;
	
	$test =  "us city (2)";	
	($lat,$long) = &uscity_latlong($locality,$region);
    }
    
    # Ill, Ore, Fla, Tex, Nev
    # Colo, Mich, Ohio, Minn, Mass, Tenn, Ariz, Utah, Wash
    # Calif, 
    # Hawaii

    elsif (($localname =~ /^([\w\s]+)\s\((.*)\)$/) && 
	   (my $state = &usstate_iso($2))) {

	my $city  = $1;

	$test =  "us city (2)";	
	($lat,$long) = &uscity_latlong($city,$state);
    } 

    elsif (($localname =~ /^([\w\s]+)\s\((.*)\)$/) && 
	   (my $code = &cdnprovince_iso($2))) {

	$test = "canadian province (2)";

	$locality = $1;
	$region   = $code;
        $country  = "CA";

	($lat,$long) = &intlcity_latlong($locality,$country);
    }

    elsif ($localname =~ /^([\w\s]+)\s\(Eng\)$/) {
	
	$locality = $1;
	$country  = "GB";

	$test = "intl city (1)";

	($lat,$long) = &intlcity_latlong($locality,$country);
    }

    elsif ($localname =~ /^([\w\s]+)\s\((.*)\)$/) {
	
	my $locality = $1;
	my $country  = $2;

	$test = "intl city (2)";

	($lat,$long) = &intlcity_latlong($locality,$country);
    }
    
    elsif ($localname eq "New York City") {

	$locality = $localname;
	$region   = "NY";
	$country  = "US";

	($lat,$long) = &uscity_latlong("New York","NY");
    }

    elsif ($localname eq "Jerusalem") {
	# don't shoot the messenger - this is
	# what Heanevn's Above thinks...

	$locality = $localname;
	$country  = "IL";

	($lat,$long) = &intlcity_latlong($localname,"IL");
    }
    
    else {
	print warn "unknown location '$name', using test '$test'";
    } 

    $country = uc(country_code2code($country,
				    LOCALE_CODE_ALPHA_2,
				    LOCALE_CODE_ALPHA_3));

    #

    if (($lat) && ($long)) {
	return {name      => $name,
		country   => $country,
		region    => $region,
		locality  => $locality,
		latitude  => $lat,
		longitude => $long};
    }

    return undef;
}

=head1 VERSION

1.0

=head1 DATE

$Date: 2004/09/18 22:07:11 $

=head1 AUTHOR

Aaron Straup Cope

=head1 LICENSE

Copyright (c) 2004 Aaron Straup Cope. All rights reserved.

This is free software, you may use it and distribute it under
the same terms as Perl itself.

=cut

return 1;
